@@ -247,8 +247,12 @@ static void sh_kern_push2db (char * name, unsigned long addr,
       tmpFile.linkpath[0] = '-'; tmpFile.linkpath[1] = '\0';
     }
 
-  sh_hash_pushdata (&tmpFile, 
-		    _("000000000000000000000000000000000000000000000000"));
+  if (sh.flag.checkSum == SH_CHECK_CHECK && sh.flag.update == S_TRUE)
+    sh_hash_pushdata_memory (&tmpFile, 
+			     _("000000000000000000000000000000000000000000000000"));
+  else
+    sh_hash_pushdata (&tmpFile, 
+		      _("000000000000000000000000000000000000000000000000"));
   return;
 }
 
@@ -367,7 +371,7 @@ int sh_kern_data_init ()
 				    &datasize, 1);
       sh_syscalls[i].addr = store0;
       if (store0 == 0) {
-	sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, 0, MSG_E_SUBGEN,
+	sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, i, MSG_E_SUBGEN,
 			_("syscall address not found in database"), 
 			_("sh_kern_data_init"));
 	return -1;
@@ -376,7 +380,7 @@ int sh_kern_data_init ()
       sh_syscalls[i].code[0] = (unsigned int) store1; 
       sh_syscalls[i].code[1] = (unsigned int) store2;
       if ((store1 == 0) || (store2 == 0)) {
-	sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, 0, MSG_E_SUBGEN,
+	sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, i, MSG_E_SUBGEN,
 			_("syscall code not found in database"), 
 			_("sh_kern_data_init"));
 	/* fprintf(stderr, "Syscall #%d\n", i); */
@@ -402,7 +406,7 @@ int sh_kern_data_init ()
 	    memcpy(&idt_table[j*8], databuf, 8);
 	    SH_FREE(databuf);
 	  } else {
-	    sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, 0, MSG_E_SUBGEN,
+	    sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, j, MSG_E_SUBGEN,
 			    _("idt table not found in database"), 
 			    _("sh_kern_data_init"));
 	    return -1;
@@ -426,6 +430,7 @@ struct inode_operations {
   int (*mknod) (int *,int *,int,int);
   int (*rename) (int *, int *,
                  int *, int *);
+  /* flawfinder: ignore */
   int (*readlink) (int *, char *,int);
   int (*follow_link) (int *, int *);
   void (*truncate) (int *);
@@ -539,7 +544,7 @@ int sh_kern_check_internal ()
   
   if (is_init == 0)
     {
-      if (sh.flag.checkSum != SH_CHECK_INIT)
+      if (sh.flag.checkSum != SH_CHECK_INIT && sh.flag.update != S_TRUE)
 	{
 	  if (0 == sh_kern_data_init()) {
 	    is_init = 1;
@@ -552,7 +557,8 @@ int sh_kern_check_internal ()
 	    SL_RETURN( (-1), _("sh_kern_check_internal"));
 	  }
 	}
-      else if ((sh.flag.checkSum == SH_CHECK_INIT) && 
+      else if ((sh.flag.checkSum == SH_CHECK_INIT || 
+		sh.flag.checkSum == SH_CHECK_CHECK) && 
 	       (sh.flag.update == S_TRUE))
 	{
 	  if (0 == sh_kern_data_init()) {
@@ -644,7 +650,8 @@ int sh_kern_check_internal ()
 		      sh_syscalls[j].addr == 0UL)
 		    break;
 
-		  if ((sh.flag.checkSum == SH_CHECK_INIT) && 
+		  if ((sh.flag.checkSum == SH_CHECK_INIT || 
+		       sh.flag.checkSum == SH_CHECK_CHECK) && 
 		      (sh.flag.update == S_TRUE))
 		    {
 		      lseek (kd, kmem_call_table[j], SEEK_SET);
@@ -852,7 +859,7 @@ int sh_kern_check_internal ()
    *
    * Stored(old) is system_call_code[]
    */
-  if (sh.flag.checkSum == SH_CHECK_INIT)
+  if (sh.flag.checkSum == SH_CHECK_INIT || sh.flag.update == S_TRUE)
     {
       store0 = 0; store1 = 0; store2 = 0;
       datasize = sizeof(system_call_code);
@@ -928,7 +935,7 @@ int sh_kern_check_internal ()
    *
    * Stored(old) is sh_syscalls[] array.
    */
-  if (sh.flag.checkSum == SH_CHECK_INIT)
+  if (sh.flag.checkSum == SH_CHECK_INIT || sh.flag.update == S_TRUE)
     {
       for (i = 0; i < SH_MAXCALLS; ++i) 
 	{
@@ -1118,7 +1125,7 @@ int sh_kern_check_internal ()
 #ifdef SH_IDT_TABLE
   if (ShKernIDT == S_TRUE)
     {
-      if (sh.flag.checkSum == SH_CHECK_INIT)
+      if (sh.flag.checkSum == SH_CHECK_INIT || sh.flag.update == S_TRUE)
 	{
 	  datasize = 8;
 	  for (j = 0; j < SH_MAXIDT; ++j) 
@@ -1351,7 +1358,7 @@ int sh_kern_check_internal ()
 
   if (is_init == 0)
     { 
-      if (sh.flag.checkSum != SH_CHECK_INIT)
+      if (sh.flag.checkSum != SH_CHECK_INIT && sh.flag.update != S_TRUE)
 	{
 	  if (0 == sh_kern_data_init()) {
 	    is_init = 1;
@@ -1364,7 +1371,8 @@ int sh_kern_check_internal ()
 	    SL_RETURN( (-1), _("sh_kern_check_internal"));
 	  }
 	}
-      else if ((sh.flag.checkSum == SH_CHECK_INIT) && 
+      else if ((sh.flag.checkSum == SH_CHECK_INIT ||
+		sh.flag.checkSum == SH_CHECK_CHECK) && 
 	       (sh.flag.update == S_TRUE))
 	{	
 	  if (0 == sh_kern_data_init()) {
@@ -1439,7 +1447,7 @@ int sh_kern_check_internal ()
    *
    * Stored(old) is sh_syscalls[] array.
    */
-  if (sh.flag.checkSum == SH_CHECK_INIT)
+  if (sh.flag.checkSum == SH_CHECK_INIT || sh.flag.update == S_TRUE)
     {
       for (i = 0; i < SH_MAXCALLS; ++i) 
 	{
@@ -1702,6 +1710,9 @@ int sh_kern_end ()
  *************/
 int sh_kern_timer (time_t tcurrent)
 {
+  if (ShKernActive == S_FALSE)
+    return 0;
+
   if ((int) (tcurrent - lastcheck) >= ShKernInterval)
     {
       lastcheck  = tcurrent;
